/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_BSPOP_HYBRID_COMMUNICATION_EFFICENCY_TEST_H
#define CUBELIB_BSPOP_HYBRID_COMMUNICATION_EFFICENCY_TEST_H

#include <string>
#include "PerformanceTest.h"

namespace bscanalysis
{
class BSPOPHybridOMPCommunicationEfficiencyTest;
class BSPOPHybridCommunicationEfficiencyTest : public popcalculation::PerformanceTest
{
    friend class BSPOPHybridOMPCommunicationEfficiencyTest;
private:

    cube::Metric* max_runtime;
    cube::Metric* max_comp;

    cube::list_of_metrics max_comp_metrics;
protected:

    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "Hybrid Communication Efficiency can be evaluated directly by following formula\n"
            "\tHybrid Communication Efficiency= max(computation time)/max(runtime)\n"
            "This metric identifies when code is inefficient because it spends a large amount \n"
            "of time communicating rather than performing useful computations.";

        std::string not_active_text =
            "Hybrid Load Balance Efficiency metric is a basic POP metric and is available for every Score-P/Scalasca measurement. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca, it might have missing metric Time.\n"
            "In this case POP analysis is not possible.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;


public:
    BSPOPHybridCommunicationEfficiencyTest(
        cube::CubeProxy* );

    virtual
    ~BSPOPHybridCommunicationEfficiencyTest()
    {
    };

    virtual
    double
    analyze( const cube::list_of_cnodes& cnodes,
             cube::LocationGroup*        _lg = nullptr ) const;

// ------ overview tests ---------

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false  );


    bool
    isActive() const;

    bool
    isIssue() const;
};
};
#endif // ADVISER_RATING_WIDGET_H
