/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_BSPOP_HYBRID_PARALLEL_EFFICIENCY_TEST_H
#define CUBELIB_BSPOP_HYBRID_PARALLEL_EFFICIENCY_TEST_H

#include <string>
#include "PerformanceTest.h"
#include "BSPOPHybridCommunicationEfficiencyTest.h"
#include "BSPOPHybridLoadBalanceTest.h"

namespace bscanalysis
{
class BSPOPHybridOMPParallelEfficiencyTest;
class BSPOPHybridParallelEfficiencyTest : public popcalculation::PerformanceTest
{
    friend class BSPOPHybridOMPParallelEfficiencyTest;
private:
    cube::Metric* pop_avg_comp;
    cube::Metric* max_runtime;

    cube::list_of_metrics max_runtime_metrics;

    void
    calculate();

    void
    calculate( const cube::list_of_cnodes& );

protected:


    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "Hybrid Parallel Efficiency (HPE) reveals the inefficiency in processes and threads utilization. \n"
            "These are measured with Hybrid Load Balance Efficiency and Hybrid Communication Efficiency, \n"
            "and HPE can be computed directly or as a product of these two sub-metrics:\n"
            "\tHPE = avg(computation time)/max(runtime) \n"
            "\t    = Hybrid Load Balance Efficiency x Hybrid Communication Efficiency";

        std::string not_active_text =
            "Hybrid Parallel Efficiency (HPE) metric is a basic POP metric and is available for every Score-P/Scalasca measurement. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca, it might have missing metric Time.\n"
            "In this case POP analysis is not possible.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;


public:
    BSPOPHybridParallelEfficiencyTest(  cube::CubeProxy*  );

    virtual
    ~BSPOPHybridParallelEfficiencyTest()
    {
    };
// ------ overview tests ---------

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false  );


    bool
    isActive() const;

    bool
    isIssue() const;
};
};
#endif // ADVISER_RATING_WIDGET_H
