/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2024                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_GPU_POP_COMPUTATION_TIME_H
#define CUBELIB_GPU_POP_COMPUTATION_TIME_H

#include <string>
#include "PerformanceTest.h"


namespace popcalculation
{
class POPGPUComputationTime : public PerformanceTest
{
private:
    cube::Metric* gpu_comp;
    bool          is_active;

protected:

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false   );

    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "GPU Computation time shows the time spent in the part of the code, identified as a useful computation and executed on GPU.\n"
            "GPU Computation time is defined as a subset of the general Computation time, namely it is the part of the Computation time,\n"
            "spent on GPU \n";
        std::string not_active_text =
            "GPU Computation time metric is available for every Score-P/Scalasca measurement, where Time metric and GPUs are present. \n"
            "If Cube Report was produced by a tool other than Score-P/Scalasca, the metrics \"Time\" and therefore the \"Computation Time\" may be missing.\n"
            "In this case this metric is reported as NAN.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;

public:
    POPGPUComputationTime( cube::CubeProxy* );

    virtual
    ~POPGPUComputationTime()
    {
    };
// ------ overview tests ---------


    virtual
    inline
    bool
    isPercent() const
    {
        return false;
    }

    bool
    isActive() const;

    bool
    isIssue() const;

    virtual
    inline
    bool
    isNormalized() const // returns true, if range goes from 0 to 1. false -> otherwise
    {
        return false;
    }

    inline
    virtual
    std::string
    units() const
    {
        return "sec";
    }
};
};
#endif // POP_ipc_TEST_H
